﻿using System.Collections.Generic;
using System.IO;

public class NeuralNetwork
{
    public List<Layer> layers = new List<Layer>();

    public void AddLayer(int numberOfNodes)
    {
        Layer newLayer = new Layer(numberOfNodes);
        if (layers.Count > 0)
        {
            layers[layers.Count - 1].ConnectDensely(newLayer);
        }
        layers.Add(newLayer);
    }

    public void CalcForward(double[] inputdata)
    {
        layers[0].SetInputData(inputdata);

        for (int i = 0; i < layers.Count; i++)
        {
            layers[i].CalcForward();
        }
    }

    public int GetMaxOutput()
    {
        int max = 0; ;
        double maxValue = 0.0;
        for (int i = 0; i < layers[layers.Count - 1].nodes.Count; i++)
        {
            if (layers[layers.Count - 1].nodes[i].value > maxValue)
            {
                max = i;
                maxValue = layers[layers.Count - 1].nodes[i].value;
            }
        }
        return max;
    }

    public void InitWeight()
    {
        foreach (Layer layer in layers)
        {
            layer.InitWeight();
        }
    }

    public void UpdateWeight(double alpha)
    {
        foreach (Layer layer in layers)
        {
            layer.UpdateWeight(alpha);
        }
    }

    public void CalcError(double[] trainData)
    {
        int numOutput = layers[layers.Count - 1].nodes.Count;

        for (int i = 0; i < numOutput; i++)
        {
            layers[layers.Count - 1].nodes[i].CalcError(trainData[i]);
        }

        for (int i = layers.Count - 2; i >= 0; i--)
        {
            foreach (Node node in layers[i].nodes)
            {
                node.CalcError();
            }
        }
    }


    public void SaveWeight()
    {
        using (FileStream fsWeights = new FileStream(@"weight.dat", FileMode.OpenOrCreate))
        {
            using (BinaryWriter bwWeights = new BinaryWriter(fsWeights))
            {
                bwWeights.Write(layers.Count);
                foreach (Layer layer in layers)
                {
                    bwWeights.Write(layer.nodes.Count);
                }

                foreach (Layer layer in layers)
                {
                    foreach (Node node in layer.nodes)
                    {
                        foreach (Edge edge in node.inputs)
                        {
                            bwWeights.Write(edge.weight);
                        }
                    }
                }
            }
        }
    }

    public void LoadWeight()
    {
        using (FileStream fsWeights = new FileStream(@"weight.dat", FileMode.Open))
        {
            using (BinaryReader brWeights = new BinaryReader(fsWeights))
            {
                layers = new List<Layer>();
                int numLayers = brWeights.ReadInt32();
                for (int i = 0; i < numLayers; i++)
                {
                    int numNodes = brWeights.ReadInt32();
                    AddLayer(numNodes);
                }

                foreach (Layer layer in layers)
                {
                    foreach (Node node in layer.nodes)
                    {
                        foreach (Edge edge in node.inputs)
                        {
                            edge.weight = brWeights.ReadDouble();
                        }
                    }
                }
            }
        }
    }
}
