﻿using System;
using System.Collections.Generic;

public class Edge
{
    public Node left;
    public Node right;
    public double weight;
}

public class Node
{
    public List<Edge> inputs = new List<Edge>();
    public List<Edge> outputs = new List<Edge>();
    public double inValue;
    public double value;
    public double error;

    static Random random = new Random();

    // 活性化関数
    public double Activation(double val)
    {
        return 1.0 / (1.0 + Math.Exp(-val));
    }

    // 活性化関数の微分
    public double DActivation(double val)
    {
        return (1.0 - val) * val;
    }

    // 隣のノードと接続する
    public Edge Connect(Node right)
    {
        Edge edge = new Edge();
        edge.left = this;
        edge.right = right;
        right.inputs.Add(edge);
        this.outputs.Add(edge);
        return edge;
    }

    // 出力値計算
    public void CalcForward()
    {
        if (inputs.Count == 0) return;

        inValue = 0.0;
        foreach (Edge edge in inputs)
        {
            inValue += edge.left.value * edge.weight;
        }

        value = Activation(inValue);
    }

    // 正規分布乱数
    public static double GetRandom()
    {
        double r1 = random.NextDouble();
        double r2 = random.NextDouble();
        return (Math.Sqrt(-2.0 * Math.Log(r1)) * Math.Cos(2.0 * Math.PI * r2)) * 0.1;
    }

    // 重みを乱数で初期化
    public void InitWeight()
    {
        foreach (Edge edge in inputs)
        {
            edge.weight = GetRandom();
        }
    }

    // 誤差計算（出力層）
    public void CalcError(double t)
    {
        error = t - value;
    }

    // 誤差計算（隠れ層）
    public void CalcError()
    {
        error = 0.0;
        foreach(Edge edge in outputs)
        {
            error += edge.weight * edge.right.error;
        }
    }

    // 重み更新
    public void UpdateWeight(double alpha)
    {
        foreach (Edge edge in inputs)
        {
            edge.weight += alpha * error * DActivation(value) * edge.left.value;
        }
    }

}
